@@ -159,11 +159,13 @@ IntegrationData <- setClass(
 #' and the default is RNA
 #' @slot model A formula used in SCTransform
 #' @slot arguments other information used in SCTransform
+#' @slot median_umi Median UMI (or scale factor) used to calculate corrected counts
 #'
 #' @seealso \code{\link{Assay}}
 #'
 #' @name SCTAssay-class
 #' @rdname SCTAssay-class
+#' @concept objects
 #'
 #' @examples
 #' \dontrun{
@@ -179,7 +181,8 @@ SCTModel <- setClass(
     clips = 'list',
     umi.assay = 'character',
     model = 'character',
-    arguments = "list"
+    arguments = 'list',
+    median_umi = 'numeric'
   )
 )
 
@@ -194,6 +197,7 @@ SCTModel <- setClass(
 #'
 #' @name SCTAssay-class
 #' @rdname SCTAssay-class
+#' @concept objects
 #'
 #' @examples
 #' # SCTAssay objects are generated from SCTransform
@@ -218,6 +222,7 @@ SCTAssay <- setClass(
 #'
 #' @rdname ScaleFactors
 #' @concept objects
+#' @concept spatial
 #' @export
 #'
 scalefactors <- function(spot, fiducial, hires, lowres) {
@@ -239,7 +244,7 @@ setOldClass(Classes = c('scalefactors'))
 #'
 #' @inheritSection SeuratObject::SpatialImage Slots
 #' @slot coordinates ...
-#' @slot ...
+#' @concept spatial
 #'
 SlideSeq <- setClass(
   Class = 'SlideSeq',
@@ -253,7 +258,8 @@ SlideSeq <- setClass(
 #'
 #'
 #' @inheritSection SeuratObject::SpatialImage Slots
-#' @slot ...
+#' @concept objects
+#' @concept spatial
 #'
 STARmap <- setClass(
   Class = 'STARmap',
@@ -279,6 +285,7 @@ STARmap <- setClass(
 #' @name VisiumV1-class
 #' @rdname VisiumV1-class
 #' @concept objects
+#' @concept spatial
 #' @exportClass VisiumV1
 #'
 VisiumV1 <- setClass(
@@ -306,7 +313,6 @@ setClass(Class = 'SliceImage', contains = 'VisiumV1')
 #' named by image name.
 #'
 #' @return A vector of cell names
-#' @concept objects
 #'
 #' @examples
 #' \dontrun{
@@ -345,7 +351,7 @@ CellsByImage <- function(object, images = NULL, unlist = FALSE) {
 #' @importFrom Matrix colSums rowSums
 #'
 #' @export
-#'
+#' @concept objects
 #'
 CreateSCTAssayObject <- function(
   counts,
@@ -448,6 +454,7 @@ CreateSCTAssayObject <- function(
 #' remove all DimReducs)
 #' @param graphs Only keep a subset of Graphs specified here (if NULL, remove
 #' all Graphs)
+#' @param misc Preserve the \code{misc} slot; default is \code{TRUE}
 #'
 #' @export
 #' @concept objects
@@ -460,7 +467,8 @@ DietSeurat <- function(
   features = NULL,
   assays = NULL,
   dimreducs = NULL,
-  graphs = NULL
+  graphs = NULL,
+  misc = TRUE
 ) {
   object <- UpdateSlots(object = object)
   assays <- assays %||% FilterObjects(object = object, classes.keep = "Assay")
@@ -502,6 +510,11 @@ DietSeurat <- function(
       }
     }
   }
+  # remove misc when desired
+  if (!isTRUE(x = misc)) {
+    slot(object = object, name = "misc") <- list()
+  }
+
   # remove unspecified DimReducs and Graphs
   all.objects <- FilterObjects(object = object, classes.keep = c('DimReduc', 'Graph'))
   objects.to.remove <- all.objects[!all.objects %in% c(dimreducs, graphs)]
@@ -533,6 +546,7 @@ DietSeurat <- function(
 #' circular filter
 #'
 #' @concept objects
+#' @concept spatial
 #' @examples
 #' \dontrun{
 #' # This example uses the ssHippo dataset which you can download
@@ -1009,7 +1023,7 @@ as.Seurat.CellDataSet <- function(
 #' set to \code{NULL} if only normalized data are present
 #' @param data name of the SingleCellExperiment assay to slot as \code{data}.
 #' Set to NULL if only counts are present
-#' @param assay Name to store expression matrices as
+#' @param assay Name of assays to convert; set to \code{NULL} for all assays to be converted
 #' @param project Project name for new Seurat object
 #'
 #' @rdname as.Seurat
@@ -1021,94 +1035,144 @@ as.Seurat.SingleCellExperiment <- function(
   x,
   counts = 'counts',
   data = 'logcounts',
-  assay = 'RNA',
+  assay = NULL,
   project = 'SingleCellExperiment',
   ...
 ) {
   CheckDots(...)
   if (!PackageCheck('SingleCellExperiment', error = FALSE)) {
     stop(
-      "Please install SingleCellExperiment from Bioconductor before converting to a SingeCellExperiment object",
+      "Please install SingleCellExperiment from Bioconductor before converting to a SingeCellExperiment object.",
+      "\nhttps://bioconductor.org/packages/SingleCellExperiment/",
       call. = FALSE
     )
   }
   meta.data <- as.data.frame(x = SummarizedExperiment::colData(x = x))
-  # Pull expression matrices
-  mats <- list(counts = counts, data = data)
-  mats <- Filter(f = Negate(f = is.null), x = mats)
-  if (length(x = mats) == 0) {
-    stop("Cannot pass 'NULL' to both 'counts' and 'data'")
-  }
-  for (m in 1:length(x = mats)) {
-    # if (is.null(x = mats[[m]])) next
-    mats[[m]] <- tryCatch(
-      expr = SummarizedExperiment::assay(x = x, i = mats[[m]]),
-      error = function(e) {
-        stop("No data in provided assay - ", mats[[m]], call. = FALSE)
-      }
-    )
-    # if cell names are NULL, fill with cell_X
-    if (is.null(x = colnames(x = mats[[m]]))) {
-      warning(
-        "The column names of the",
-        names(x = mats)[m],
-        " matrix is NULL. Setting cell names to cell_columnidx (e.g 'cell_1').",
-        call. = FALSE,
-        immediate. = TRUE
-      )
-      cell.names <- paste0("cell_", 1:ncol(x = mats[[m]]))
-      colnames(x = mats[[m]]) <- cell.names
-      rownames(x = meta.data) <- cell.names
-    }
-  }
-  assays <- if (is.null(x = mats$counts)) {
-    list(CreateAssayObject(data = mats$data))
-  } else if (is.null(x = mats$data)) {
-    list(CreateAssayObject(counts = mats$counts))
+  if (packageVersion(pkg = "SingleCellExperiment") >= "1.14.0") {
+    orig.exp <- SingleCellExperiment::mainExpName(x = x) %||% "originalexp"
   } else {
-    a <- CreateAssayObject(counts = mats$counts)
-    a <- SetAssayData(object = a, slot = 'data', new.data = mats$data)
-    list(a)
+    orig.exp <- "originalexp"
   }
-  names(x = assays) <- assay
-  Key(object = assays[[assay]]) <- paste0(tolower(x = assay), '_')
-  # Create the Seurat object
-  object <- new(
-    Class = 'Seurat',
-    assays = assays,
-    meta.data = meta.data,
-    version = packageVersion(pkg = 'Seurat'),
-    project.name = project
-  )
-  DefaultAssay(object = object) <- assay
-  Idents(object = object) <- project
-  # Get DimReduc information
-  if (length(x = SingleCellExperiment::reducedDimNames(x = x)) > 0) {
-    for (dr in SingleCellExperiment::reducedDimNames(x = x)) {
-      embeddings <- SingleCellExperiment::reducedDim(x = x, type = dr)
-      if (is.null(x = rownames(x = embeddings))) {
-        rownames(x = embeddings)  <- cell.names
-      }
-      key <- gsub(
-        pattern = "[[:digit:]]",
-        replacement = "_",
-        x = colnames(x = SingleCellExperiment::reducedDim(x = x, type = dr))[1]
+  if (!is.null(SingleCellExperiment::altExpNames(x = x))) {
+    assayn <- assay %||% SingleCellExperiment::altExpNames(x = x)
+    if (!all(assay %in% SingleCellExperiment::altExpNames(x = x))) {
+      stop("One or more of the assays you are trying to convert is not in the SingleCellExperiment object")
+    }
+    assayn <- c(orig.exp, assayn)
+  } else {
+    assayn <- orig.exp
+  }
+  for (assay in assayn) {
+    if (assay != orig.exp) {
+      x <- SingleCellExperiment::swapAltExp(x = x, name = assay, saved = NULL)
+    }
+    # Pull matrices
+    mats <- list(counts = counts, data = data)
+    mats <- Filter(f = Negate(f = is.null), x = mats)
+    if (length(x = mats) == 0) {
+      stop("Cannot pass 'NULL' to both 'counts' and 'data'")
+    }
+    for (m in 1:length(x = mats)) {
+      mats[[m]] <- tryCatch(
+        expr = SummarizedExperiment::assay(x = x, i = mats[[m]]),
+        error = function(e) {
+          stop("No data in provided assay - ", mats[[m]], call. = FALSE)
+        }
       )
-      if (length(x = key) == 0) {
-        key <- paste0(dr, "_")
+      # if cell names are NULL, fill with cell_X
+      if (is.null(x = colnames(x = mats[[m]]))) {
+        warning(
+          "The column names of the ",
+          names(x = mats)[m],
+          " matrix is NULL. Setting cell names to cell_columnidx (e.g 'cell_1').",
+          call. = FALSE,
+          immediate. = TRUE
+        )
+        cell.names <- paste0("cell_", 1:ncol(x = mats[[m]]))
+        colnames(x = mats[[m]]) <- cell.names
+        rownames(x = meta.data) <- cell.names
       }
-      colnames(x = embeddings) <- paste0(key, 1:ncol(x = embeddings))
-      object[[dr]] <- CreateDimReducObject(
-        embeddings = embeddings,
-        key = key,
-        assay = DefaultAssay(object = object)
+    }
+    assays <- if (is.null(x = mats$counts)) {
+      list(CreateAssayObject(data = mats$data))
+    } else if (is.null(x = mats$data)) {
+      list(CreateAssayObject(counts = mats$counts))
+    } else {
+      a <- CreateAssayObject(counts = mats$counts)
+      a <- SetAssayData(object = a, slot = 'data', new.data = mats$data)
+      list(a)
+    }
+    names(x = assays) <- assay
+    Key(object = assays[[assay]]) <- paste0(tolower(x = assay), '_')
+    # Create the Seurat object
+    if (!exists(x = "object")) {
+      object <- CreateSeuratObject(
+        counts = assays[[assay]],
+        Class = 'Seurat',
+        assay = assay,
+        meta.data = meta.data,
+        version = packageVersion(pkg = 'Seurat'),
+        project.name = project
       )
+    } else {
+      object[[assay]] <- assays[[assay]]
+    }
+    DefaultAssay(object = object) <- assay
+    # add feature level meta data
+    md <- SingleCellExperiment::rowData(x = x)
+    if (ncol(x = md) > 0) {
+      # replace underscores
+      rownames(x = md) <- gsub(pattern = "_", replacement = "-", x = rownames(x = md))
+      md <- as.data.frame(x = md)
+      # ensure order same as data
+      md <- md[rownames(x = object[[assay]]), , drop = FALSE]
+      object[[assay]] <- AddMetaData(
+        object = object[[assay]],
+        metadata = md
+      )
+    }
+    Idents(object = object) <- project
+    # Get DimReduc information, add underscores if needed and pull from different alt EXP
+    if (length(x = SingleCellExperiment::reducedDimNames(x = x)) > 0) {
+      for (dr in SingleCellExperiment::reducedDimNames(x = x)) {
+        embeddings <- as.matrix(x = SingleCellExperiment::reducedDim(x = x, type = dr))
+        if (is.null(x = rownames(x = embeddings))) {
+          rownames(x = embeddings)  <- cell.names
+        }
+        if (isTRUE(x = !grepl('_$',
+        gsub(pattern = "[[:digit:]]",
+          replacement = "_",
+          x = colnames(x = SingleCellExperiment::reducedDim(x = x, type = dr))[1]
+        )))) {
+        key <- gsub(
+          pattern = "[[:digit:]]",
+          replacement = "_",
+          x = colnames(x = SingleCellExperiment::reducedDim(x = x, type = dr))[1]
+        )
+        } else
+        {
+          key <- gsub(
+            pattern = "[[:digit:]]",
+            replacement = "",
+            x = colnames(x = SingleCellExperiment::reducedDim(x = x, type = dr))[1]
+          )
+       }
+        if (length(x = key) == 0) {
+          key <- paste0(dr, "_")
+        }
+        colnames(x = embeddings) <- paste0(key, 1:ncol(x = embeddings))
+        object[[dr]] <- CreateDimReducObject(
+          embeddings = embeddings,
+          key = key,
+          assay = DefaultAssay(object = object)
+        )
+      }
     }
   }
   return(object)
 }
 
-#' @param assay Assay to convert
+#' @param assay Assays to convert
 #'
 #' @rdname as.SingleCellExperiment
 #' @concept objects
@@ -1120,19 +1184,80 @@ as.SingleCellExperiment.Seurat <- function(x, assay = NULL, ...) {
   if (!PackageCheck('SingleCellExperiment', error = FALSE)) {
     stop("Please install SingleCellExperiment from Bioconductor before converting to a SingeCellExperiment object")
   }
-  assay <- assay %||% DefaultAssay(object = x)
-  assays = list(
-    counts = GetAssayData(object = x, assay = assay, slot = "counts"),
-    logcounts = GetAssayData(object = x, assay = assay, slot = "data")
-  )
-  assays <- assays[sapply(X = assays, FUN = nrow) != 0]
-  sce <- SingleCellExperiment::SingleCellExperiment(assays = assays)
+  assay <- assay %||% Assays(object = x)
+  if (!all(assay %in% Assays(object = x))) {
+    stop("One or more of the assays you are trying to convert is not in the Seurat object")
+  }
+  if (DefaultAssay(object = x) %in% assay) {
+    assay <- union(DefaultAssay(object = x), assay)
+  }
+  experiments <- list()
+  for (assayn in assay) {
+    assays <- list(
+      counts = GetAssayData(object = x, assay = assayn, slot = "counts"),
+      logcounts = GetAssayData(object = x, assay = assayn, slot = "data")
+    )
+    scaledata_a <- GetAssayData(object = x, assay = assayn, slot = "scale.data")
+    if (isTRUE(x = all.equal(
+      target = dim(x = assays[["counts"]]),
+      current = dim(x = scaledata_a))
+    )) {
+      assays[["scaledata"]] <- scaledata_a
+    }
+    assays <- assays[sapply(X = assays, FUN = nrow) != 0]
+    sume <- SummarizedExperiment::SummarizedExperiment(assays = assays)
+    experiments[[assayn]] <- sume
+  }
+  # create one single cell experiment
+  sce <- as(object = experiments[[1]], Class = "SingleCellExperiment")
+  orig.exp.name <- names(x = experiments[1])
+  if (packageVersion(pkg = "SingleCellExperiment") >= "1.14.0") {
+    SingleCellExperiment::mainExpName(sce) <- names(x = experiments[1])
+  }
+  if (length(x = experiments) > 1) {
+    sce <- SingleCellExperiment::SingleCellExperiment(sce, altExps = experiments)
+    sce <- SingleCellExperiment::swapAltExp(
+      x = sce,
+      name = orig.exp.name,
+      saved = NULL
+    )
+  }
   metadata <- x[[]]
   metadata$ident <- Idents(object = x)
-  SummarizedExperiment::colData(sce) <- S4Vectors::DataFrame(metadata)
-  SummarizedExperiment::rowData(sce) <- S4Vectors::DataFrame(x[[assay]][[]])
+  SummarizedExperiment::colData(x = sce) <- S4Vectors::DataFrame(metadata)
+  for (assayn in assay) {
+    if (assayn != orig.exp.name) {
+      sce <- SingleCellExperiment::swapAltExp(
+        x = sce,
+        name = assayn,
+        saved = orig.exp.name
+      )
+      SummarizedExperiment::rowData(x = sce) <- S4Vectors::DataFrame(x[[assayn]][[]])
+      sce <- SingleCellExperiment::swapAltExp(
+        x = sce,
+        name = orig.exp.name,
+        saved = assayn
+      )
+    }
+  }
   for (dr in FilterObjects(object = x, classes.keep = "DimReduc")) {
-    SingleCellExperiment::reducedDim(sce, toupper(x = dr)) <- Embeddings(object = x[[dr]])
+    assay.used <- DefaultAssay(object = x[[dr]])
+    swap.exp <- assay.used %in% SingleCellExperiment::altExpNames(x = sce) & assay.used != orig.exp.name
+    if (swap.exp) {
+      sce <- SingleCellExperiment::swapAltExp(
+        x = sce,
+        name = assay.used,
+        saved = orig.exp.name
+      )
+    }
+    SingleCellExperiment::reducedDim(x = sce, type = toupper(x = dr)) <- Embeddings(object = x[[dr]])
+    if (swap.exp) {
+      sce <- SingleCellExperiment::swapAltExp(
+        x = sce,
+        name = orig.exp.name,
+        saved = assay.used
+      )
+    }
   }
   return(sce)
 }
@@ -1183,27 +1308,29 @@ as.sparse.H5Group <- function(x, ...) {
 #' @method Cells SCTModel
 #' @export
 #'
-Cells.SCTModel <- function(x) {
+Cells.SCTModel <- function(x, ...) {
   return(rownames(x = slot(object = x, name = "cell.attributes")))
 }
 
 #' @rdname Cells
 #' @concept objects
+#' @concept spatial
 #' @method Cells SlideSeq
 #' @export
 #'
 #' @seealso \code{\link[SeuratObject:Cells]{SeuratObject::Cells}}
 #'
-Cells.SlideSeq <- function(x) {
+Cells.SlideSeq <- function(x, ...) {
   return(rownames(x = GetTissueCoordinates(object = x)))
 }
 
 #' @rdname Cells
 #' @concept objects
+#' @concept spatial
 #' @method Cells STARmap
 #' @export
 #'
-Cells.STARmap <- function(x) {
+Cells.STARmap <- function(x, ...) {
   return(rownames(x = GetTissueCoordinates(object = x)))
 }
 
@@ -1212,7 +1339,7 @@ Cells.STARmap <- function(x) {
 #' @method Cells VisiumV1
 #' @export
 #'
-Cells.VisiumV1 <- function(x) {
+Cells.VisiumV1 <- function(x, ...) {
   return(rownames(x = GetTissueCoordinates(object = x, scale = NULL)))
 }
 
@@ -1250,6 +1377,7 @@ GetAssay.Seurat <- function(object, assay = NULL, ...) {
 #' @rdname GetImage
 #' @method GetImage SlideSeq
 #' @concept objects
+#' @concept spatial
 #' @export
 #'
 #' @seealso \code{\link[SeuratObject:GetImage]{SeuratObject::GetImage}}
@@ -1266,6 +1394,7 @@ GetImage.SlideSeq <- function(
 #' @rdname GetImage
 #' @method GetImage STARmap
 #' @concept objects
+#' @concept spatial
 #' @export
 #'
 GetImage.STARmap <- function(
@@ -1283,6 +1412,7 @@ GetImage.STARmap <- function(
 #'
 #' @rdname GetImage
 #' @concept objects
+#' @concept spatial
 #' @method GetImage VisiumV1
 #' @export
 #'
@@ -1326,6 +1456,7 @@ GetImage.VisiumV1 <- function(
 #' @rdname GetTissueCoordinates
 #' @method GetTissueCoordinates SlideSeq
 #' @concept objects
+#' @concept spatial
 #' @export
 #'
 #' @seealso \code{\link[SeuratObject:GetTissueCoordinates]{SeuratObject::GetTissueCoordinates}}
@@ -1344,6 +1475,7 @@ GetTissueCoordinates.SlideSeq <- function(object, ...) {
 #' @rdname GetTissueCoordinates
 #' @method GetTissueCoordinates STARmap
 #' @concept objects
+#' @concept spatial
 #' @export
 #'
 GetTissueCoordinates.STARmap <- function(object, qhulls = FALSE, ...) {
@@ -1360,6 +1492,7 @@ GetTissueCoordinates.STARmap <- function(object, qhulls = FALSE, ...) {
 #' @rdname GetTissueCoordinates
 #' @method GetTissueCoordinates VisiumV1
 #' @concept objects
+#' @concept spatial
 #' @export
 #'
 GetTissueCoordinates.VisiumV1 <- function(
@@ -1380,12 +1513,49 @@ GetTissueCoordinates.VisiumV1 <- function(
   return(coordinates)
 }
 
+#' Get Variable Feature Information
+#'
+#' Get variable feature information from \code{\link{SCTAssay}} objects
+#'
+#' @inheritParams SeuratObject::HVFInfo
+#'
+#' @export
+#' @method HVFInfo SCTAssay
+#'
+#' @seealso \code{\link[SeuratObject]{HVFInfo}}
+#'
+#' @examples
+#' # Get the HVF info directly from an SCTAssay object
+#' pbmc_small <- SCTransform(pbmc_small)
+#' HVFInfo(pbmc_small[["SCT"]], selection.method = 'sct')[1:5, ]
+#'
+HVFInfo.SCTAssay <- function(object, selection.method, status = FALSE, ...) {
+  CheckDots(...)
+  disp.methods <- c('mean.var.plot', 'dispersion', 'disp')
+  if (tolower(x = selection.method) %in% disp.methods) {
+    selection.method <- 'mvp'
+  }
+  selection.method <- switch(
+    EXPR = tolower(x = selection.method),
+    'sctransform' = 'sct',
+    selection.method
+  )
+  vars <- c('gmean', 'variance', 'residual_variance')
+  hvf.info <- SCTResults(object = object, slot = "feature.attributes")[,vars]
+  if (status) {
+    hvf.info$variable <- FALSE
+    hvf.info[VariableFeatures(object = object), "variable"] <-  TRUE
+  }
+  return(hvf.info)
+}
+
 #' Get Spot Radius
 #'
 #' @inheritParams SeuratObject::Radius
 #'
 #' @rdname Radius
 #' @concept objects
+#' @concept spatial
 #' @method Radius SlideSeq
 #' @export
 #'
@@ -1397,6 +1567,7 @@ Radius.SlideSeq <- function(object) {
 
 #' @rdname Radius
 #' @concept objects
+#' @concept spatial
 #' @method Radius STARmap
 #' @export
 #'
@@ -1406,6 +1577,7 @@ Radius.STARmap <- function(object) {
 
 #' @rdname Radius
 #' @concept objects
+#' @concept spatial
 #' @method Radius VisiumV1
 #' @export
 #'
@@ -1415,6 +1587,7 @@ Radius.VisiumV1 <- function(object) {
 
 #' @rdname RenameCells
 #' @export
+#' @concept objects
 #' @method RenameCells SCTAssay
 #'
 RenameCells.SCTAssay <- function(object, new.names = NULL, ...) {
@@ -1448,6 +1621,7 @@ RenameCells.SCTAssay <- function(object, new.names = NULL, ...) {
 #' @inheritParams SeuratObject::RenameCells
 #'
 #' @rdname RenameCells
+#' @concept objects
 #' @method RenameCells SlideSeq
 #' @export
 #'
@@ -1458,6 +1632,7 @@ RenameCells.SlideSeq <- function(object, new.names = NULL, ...) {
 }
 
 #' @rdname RenameCells
+#' @concept objects
 #' @method RenameCells STARmap
 #' @export
 #'
@@ -1471,6 +1646,7 @@ RenameCells.STARmap <- function(object, new.names = NULL, ...) {
 }
 
 #' @rdname RenameCells
+#' @concept objects
 #' @method RenameCells VisiumV1
 #' @export
 #'
@@ -1493,7 +1669,7 @@ RenameCells.VisiumV1 <- function(object, new.names = NULL, ...) {
 #'
 SCTResults.SCTModel <- function(object, slot, ...) {
   CheckDots(...)
-  slots.use <- c('feature.attributes', 'cell.attributes', 'clips','umi.assay',  'model', 'arguments')
+  slots.use <- c('feature.attributes', 'cell.attributes', 'clips','umi.assay',  'model', 'arguments', 'median_umi')
   if (!slot %in% slots.use) {
     stop(
       "'slot' must be one of ",
@@ -1505,11 +1681,12 @@ SCTResults.SCTModel <- function(object, slot, ...) {
 }
 
 #' @rdname SCTResults
+#' @concept objects
 #' @export
 #' @method SCTResults<- SCTModel
 #'
 "SCTResults<-.SCTModel" <- function(object, slot, ..., value) {
-  slots.use <- c('feature.attributes', 'cell.attributes', 'clips','umi.assay', 'model', 'arguments')
+  slots.use <- c('feature.attributes', 'cell.attributes', 'clips','umi.assay', 'model', 'arguments', 'median_umi')
   if (!slot %in% slots.use) {
     stop(
       "'slot' must be one of ",
@@ -1531,12 +1708,13 @@ SCTResults.SCTModel <- function(object, slot, ...) {
 #' the slot directly).
 #'
 #' @rdname SCTResults
+#' @concept objects
 #' @export
 #' @method SCTResults SCTAssay
 #'
 SCTResults.SCTAssay <- function(object, slot, model = NULL, ...) {
   CheckDots(...)
-  slots.use <- c('feature.attributes', 'cell.attributes', 'clips','umi.assay',  'model', 'arguments')
+  slots.use <- c('feature.attributes', 'cell.attributes', 'clips', 'umi.assay',  'model', 'arguments', 'median_umi')
   if (!slot %in% slots.use) {
     stop(
       "'slot' must be one of ",
@@ -1554,11 +1732,12 @@ SCTResults.SCTAssay <- function(object, slot, model = NULL, ...) {
 }
 
 #' @rdname SCTResults
+#' @concept objects
 #' @export
 #' @method SCTResults<- SCTAssay
 #'
 "SCTResults<-.SCTAssay" <- function(object, slot, model = NULL, ..., value) {
-  slots.use <- c('feature.attributes', 'cell.attributes', 'clips','umi.assay', 'model', 'arguments')
+  slots.use <- c('feature.attributes', 'cell.attributes', 'clips','umi.assay', 'model', 'arguments', 'median_umi')
   if (!slot %in% slots.use) {
     stop(
       "'slot' must be one of ",
@@ -1588,6 +1767,7 @@ SCTResults.SCTAssay <- function(object, slot, model = NULL, ...) {
 #'
 #' @rdname SCTResults
 #' @export
+#' @concept objects
 #' @method SCTResults Seurat
 #'
 SCTResults.Seurat <- function(object, assay = "SCT", slot, model = NULL, ...) {
@@ -1598,6 +1778,7 @@ SCTResults.Seurat <- function(object, assay = "SCT", slot, model = NULL, ...) {
 #' @rdname ScaleFactors
 #' @method ScaleFactors VisiumV1
 #' @export
+#' @concept spatial
 #'
 ScaleFactors.VisiumV1 <- function(object, ...) {
   return(slot(object = object, name = 'scale.factors'))
@@ -1606,6 +1787,7 @@ ScaleFactors.VisiumV1 <- function(object, ...) {
 #' @rdname ScaleFactors
 #' @method ScaleFactors VisiumV1
 #' @export
+#' @concept spatial
 #'
 ScaleFactors.VisiumV1 <- function(object, ...) {
   return(slot(object = object, name = 'scale.factors'))
@@ -1677,6 +1859,7 @@ dim.VisiumV1 <- function(x) {
 #' @return \code{levels}: SCT model names
 #'
 #' @export
+#' @concept objects
 #' @method levels SCTAssay
 #'
 #' @examples
@@ -1699,6 +1882,7 @@ levels.SCTAssay <- function(x) {
 #' @return \code{levels<-}: \code{x} with updated SCT model names
 #'
 #' @export
+#' @concept objects
 #' @method levels<- SCTAssay
 #'
 "levels<-.SCTAssay" <- function(x, value) {
@@ -1721,11 +1905,13 @@ levels.SCTAssay <- function(x) {
 #' Merge SCTAssay objects
 #'
 #' @inheritParams SeuratObject::merge
+#' @param x A \code{\link[SeuratObject]{Seurat}} object
 #' @param na.rm If na.rm = TRUE, this will only preserve residuals that are
 #' present in all SCTAssays being merged. Otherwise, missing residuals will be
 #' populated with NAs.
 #' @export
 #' @method merge SCTAssay
+#' @concept objects
 #'
 merge.SCTAssay <- function(
   x = NULL,
@@ -1861,6 +2047,7 @@ merge.SCTAssay <- function(
 #'
 #' @export
 #' @method subset AnchorSet
+#' @concept objects
 #'
 subset.AnchorSet <- function(
   x,
@@ -1922,11 +2109,14 @@ subset.AnchorSet <- function(
   }
   # Filter based on ident pairings
   if (!is.null(x = group.by)) {
-    anchors <- AnnotateAnchors(object = x, vars = group.by)
+    anchors <- AnnotateAnchors(anchors = x, vars = group.by)
     if (!is.null(x = disallowed.ident.pairs) && !is.null(x = ident.matrix)) {
       stop("Please use either disallowed.ident.pairs OR ident.matrix, not both.")
     }
-    unique.ids <- unique(x = c(anchors[, paste0("cell1.", group.by)], anchors[, paste0("cell2.", group.by)]))
+    unique.ids <- unique(x = c(
+      as.character(x = anchors[, paste0("cell1.", group.by)]),
+      as.character(x = anchors[, paste0("cell2.", group.by)]))
+    )
     unique.ids <- unique.ids[!is.na(x = unique.ids)]
     num.ids <- length(x = unique.ids)
     if (!is.null(x = disallowed.ident.pairs)) {
@@ -1963,6 +2153,7 @@ subset.AnchorSet <- function(
 
 #' @export
 #' @method subset SCTAssay
+#' @concept objects
 #'
 subset.SCTAssay <- function(x, cells = NULL, features = NULL, ...) {
   x <- NextMethod()
@@ -2008,13 +2199,12 @@ subset.VisiumV1 <- function(x, cells, ...) {
   return(x)
 }
 
-
-
 #' Update pre-V4 Assays generated with SCTransform in the Seurat to the new
 #' SCTAssay class
 #
 #' @param object A Seurat object
 #' @export
+#' @concept objects
 #' @return A Seurat object with updated SCTAssays
 #'
 UpdateSCTAssays <- function(object) {
@@ -2071,14 +2261,14 @@ setAs(
           X = 1:length(x = vst.res),
           FUN = function(i) {
             vst.res[[i]]$umi.assay <- umi.assay[[i]]
-            return(PrepVSTResults(vst.res = vst.res[[i]], cell.names = colnames(x = from)))
+            return(PrepVSTResults(
+              vst.res = vst.res[[i]],
+              cell.names = colnames(x = from)
+            ))
           }
         )
         names(x = vst.res) <- paste0("model", 1:length(x = vst.res))
       }
-      if (length(x = vst.res) > 1) {
-        vst.res <- merge(x = vst.res[[1]], y = vst.res[2:length(x = vst.res)])
-      }
       object.list$misc[[vst.use]] <- NULL
       object.list$SCTModel.list <- vst.res
     }
@@ -2092,7 +2282,7 @@ setMethod(
   definition = function(object) {
     cat('An AnchorSet object containing', nrow(x = slot(object = object, name = "anchors")),
         "anchors between the reference and query Seurat objects. \n",
-        "This can be used as input to TransferData.")
+        "This can be used as input to TransferData.\n")
   }
 )
 
@@ -2102,7 +2292,7 @@ setMethod(
   definition = function(object) {
     cat('An AnchorSet object containing', nrow(x = slot(object = object, name = "anchors")),
         "anchors between", length(x = slot(object = object, name = "object.list")), "Seurat objects \n",
-        "This can be used as input to IntegrateData.")
+        "This can be used as input to IntegrateData.\n")
   }
 )
 
@@ -2113,7 +2303,7 @@ setMethod(
     cat(
       'A ModalityWeights object containing modality weights between',
       paste(slot(object = object, name = "modality.assay"), collapse = " and "),
-      "assays \n", "This can be used as input to FindMultiModelNeighbors.")
+      "assays \n", "This can be used as input to FindMultiModelNeighbors.\n")
   }
 )
 
@@ -2125,10 +2315,12 @@ setMethod(
       "An sctransform model.\n",
       " Model formula: ", slot(object = object, name = "model"),
       "\n  Parameters stored for", nrow(x = SCTResults(object = object, slot = "feature.attributes")), "features,",
-      nrow(x = SCTResults(object = object, slot = "cell.attributes")), "cells")
+      nrow(x = SCTResults(object = object, slot = "cell.attributes")), "cells.\n")
   }
 )
 
+#' @importFrom utils head
+#
 setMethod(
   f = 'show',
   signature = 'SCTAssay',
@@ -2373,19 +2565,37 @@ PrepVSTResults <- function(vst.res, cell.names) {
       y = rownames(feature.attrs)
     )
     feature.attrs[genes_step1,"genes_log_gmean_step1"] <- TRUE
+
+    # add parameters from step1
+    feature.attrs[, paste0("step1_", colnames(vst.res$model_pars))] <- NA
+    feature.attrs[genes_step1, paste0("step1_", colnames(vst.res$model_pars))] <- vst.res$model_pars[genes_step1,]
+
   }
   # Prepare clipping information
   clips <- list(
     'vst' = vst.res$arguments$res_clip_range,
     'sct' = vst.res$arguments$sct.clip.range
   )
+  median_umi <- NA
+  # check if a custom scale_factor was provided to vst()
+  if ("scale_factor" %in% names(vst.res$arguments)){
+    median_umi <- vst.res$arguments$scale_factor
+  }
+  if (is.na(median_umi)) {
+    if ("umi" %in% colnames(x = cell.attrs)) {
+      median_umi <- median(cell.attrs$umi)
+    } else if ("log_umi" %in% colnames(x = cell.attrs)) {
+      median_umi <- median(10 ^ cell.attrs$log_umi)
+    }
+  }
   vst.res.SCTModel  <- SCTModel(
     feature.attributes = feature.attrs,
     cell.attributes = cell.attrs,
     clips = clips,
     umi.assay = vst.res$umi.assay %||% "RNA",
     model =  vst.res$model_str,
-    arguments =  vst.res$arguments
+    arguments =  vst.res$arguments,
+    median_umi = median_umi
   )
   return(vst.res.SCTModel)
 }
@@ -2446,6 +2656,8 @@ SubsetVST <- function(sct.info, cells, features) {
 # @return The top \code{num}
 # @seealso \{code{\link{TopCells}}} \{code{\link{TopFeatures}}}
 #
+#' @importFrom utils head tail
+#
 Top <- function(data, num, balanced) {
   nr <- nrow(x = data)
   if (num > nr) {
@@ -2485,10 +2697,10 @@ UpdateAssay <- function(old.assay, assay){
   counts <- old.assay@raw.data
   data <- old.assay@data
   if (!inherits(x = counts, what = 'dgCMatrix')) {
-    counts <- as(object = as.matrix(x = counts), Class = 'dgCMatrix')
+    counts <- as.sparse(x = as.matrix(x = counts))
   }
   if (!inherits(x = data, what = 'dgCMatrix')) {
-    data <- as(object = as.matrix(x = data), Class = 'dgCMatrix')
+    data <- as.sparse(x = as.matrix(x = data))
   }
   new.assay <- new(
     Class = 'Assay',
@@ -2648,6 +2860,8 @@ UpdateSlots <- function(object) {
 # @return Returns the data matrix if present (i.e.) not 0x0. Otherwise, returns an
 # appropriately sized empty sparse matrix
 #
+#' @importFrom Matrix Matrix
+#
 ValidateDataForMerge <- function(assay, slot) {
   mat <- GetAssayData(object = assay, slot = slot)
   if (any(dim(x = mat) == c(0, 0))) {
@@ -2668,7 +2882,7 @@ ValidateDataForMerge <- function(assay, slot) {
       ncol = data.dims[2],
       dimnames = dimnames(x = GetAssayData(object = assay, slot = data.slot))
     )
-    mat <- as(object = mat, Class = "dgCMatrix")
+    mat <- as.sparse(x = mat)
   }
   return(mat)
 }
