@@ -1,4 +1,5 @@
 #' @include generics.R
+#' @importFrom SeuratObject PackageCheck
 #'
 NULL
 
@@ -6,6 +7,98 @@ NULL
 # Functions
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+#' Add Azimuth Results
+#'
+#' Add mapping and prediction scores, UMAP embeddings, and imputed assay (if
+#' available)
+#' from Azimuth to an existing or new \code{\link[SeuratObject]{Seurat}} object
+#'
+#' @param object A \code{\link[SeuratObject]{Seurat}} object
+#' @param filename Path to Azimuth mapping scores file
+#'
+#' @return \code{object} with Azimuth results added
+#'
+#' @examples
+#' \dontrun{
+#' object <- AddAzimuthResults(object, filename = "azimuth_results.Rds")
+#' }
+#'
+#' @export
+AddAzimuthResults <- function(object = NULL, filename) {
+  if (is.null(x = filename)) {
+    stop("No Azimuth results provided.")
+  }
+  azimuth_results <- readRDS(file = filename)
+  if (!is.list(x = azimuth_results) || any(!(c('umap', 'pred.df') %in% names(x = azimuth_results)))) {
+    stop("Expected following format for azimuth_results:
+           `list(umap = <DimReduc>, pred.df = <data.frame>[, impADT = <Assay>])`")
+  }
+
+  if (is.null(x = object)) {
+    message("No existing Seurat object provided. Creating new one.")
+    object <- CreateSeuratObject(
+      counts = matrix(
+        nrow = 1,
+        ncol = nrow(x = azimuth_results$umap),
+        dimnames = list(
+          row.names = 'Dummy.feature',
+          col.names = rownames(x = azimuth_results$umap))
+      ),
+      assay = 'Dummy'
+    )
+  } else {
+    overlap.cells <- intersect(
+      x = Cells(x = object),
+      y = rownames(x = azimuth_results$umap)
+    )
+    if (!(all(overlap.cells %in% Cells(x = object)))) {
+      stop("Cells in object do not match cells in download")
+    } else if (length(x = overlap.cells) < length(x = Cells(x = object))) {
+      warning(paste0("Subsetting out ", length(x = Cells(x = object)) - length(x = overlap.cells),
+                     " cells that are absent in downloaded results (perhaps filtered by Azimuth)"))
+      object <- subset(x = object, cells = overlap.cells)
+    }
+  }
+
+  azimuth_results$pred.df$cell <- NULL
+  object <- AddMetaData(object = object, metadata = azimuth_results$pred.df)
+  object[['umap.proj']] <- azimuth_results$umap
+  if ('impADT' %in% names(x = azimuth_results)) {
+    object[['impADT']] <- azimuth_results$impADT
+    if ('Dummy' %in% Assays(object = object)) {
+      DefaultAssay(object = object) <- 'impADT'
+      object[['Dummy']] <- NULL
+    }
+  }
+  return(object)
+}
+
+#' Add Azimuth Scores
+#'
+#' Add mapping and prediction scores from Azimuth to a
+#' \code{\link[SeuratObject]{Seurat}} object
+#'
+#' @param object A \code{\link[SeuratObject]{Seurat}} object
+#' @param filename Path to Azimuth mapping scores file
+#'
+#' @return \code{object} with the mapping scores added
+#'
+#' @examples
+#' \dontrun{
+#' object <- AddAzimuthScores(object, filename = "azimuth_pred.tsv")
+#' }
+#'
+AddAzimuthScores <- function(object, filename) {
+  if (!file.exists(filename)) {
+    stop("Cannot find Azimuth scores file ", filename, call. = FALSE)
+  }
+  object <- AddMetaData(
+    object = object,
+    metadata = read.delim(file = filename, row.names = 1)
+  )
+  return(object)
+}
+
 #' Calculate module scores for feature expression programs in single cells
 #'
 #' Calculate the average expression levels of each program (cluster) on single
@@ -488,6 +581,7 @@ CellCycleScoring <- function(
 #' \code{ncontrols} most highly expressed features are kept, and the
 #' prefix is kept. All other rows are retained.
 #'
+#' @importFrom utils head
 #' @importFrom Matrix rowSums
 #'
 #' @export
@@ -704,14 +798,25 @@ FastRowScale <- function(
 #' @note This function requires internet access
 #'
 #' @param symbols A vector of gene symbols
-#' @param timeout Time to wait before cancelling query in seconds
-#' @param several.ok Allow several current gene sybmols for each provided symbol
+#' @param timeout Time to wait before canceling query in seconds
+#' @param several.ok Allow several current gene symbols for each
+#' provided symbol
+#' @param search.types Type of query to perform:
+#' \describe{
+#'  \item{\dQuote{\code{alias_symbol}}}{Find alternate symbols for the genes
+#'  described by \code{symbols}}
+#'  \item{\dQuote{\code{prev_symbol}}}{Find new new symbols for the genes
+#'  described by \code{symbols}}
+#' }
+#' This parameter accepts multiple options and short-hand options
+#' (eg. \dQuote{\code{prev}} for \dQuote{\code{prev_symbol}})
 #' @param verbose Show a progress bar depicting search progress
 #' @param ... Extra parameters passed to \code{\link[httr]{GET}}
 #'
-#' @return For \code{GeneSymbolThesarus}, if \code{several.ok}, a named list
-#' where each entry is the current symbol found for each symbol provided and the
-#' names are the provided symbols. Otherwise, a named vector with the same information.
+#' @return \code{GeneSymbolThesarus}:, if \code{several.ok}, a named list
+#' where each entry is the current symbol found for each symbol provided and
+#' the names are the provided symbols. Otherwise, a named vector with the
+#' same information.
 #'
 #' @source \url{https://www.genenames.org/} \url{https://www.genenames.org/help/rest/}
 #'
@@ -735,11 +840,13 @@ GeneSymbolThesarus <- function(
   symbols,
   timeout = 10,
   several.ok = FALSE,
+  search.types = c('alias_symbol', 'prev_symbol'),
   verbose = TRUE,
   ...
 ) {
   db.url <- 'http://rest.genenames.org/fetch'
-  search.types <- c('alias_symbol', 'prev_symbol')
+  # search.types <- c('alias_symbol', 'prev_symbol')
+  search.types <- match.arg(arg = search.types, several.ok = TRUE)
   synonyms <- vector(mode = 'list', length = length(x = symbols))
   not.found <- vector(mode = 'logical', length = length(x = symbols))
   multiple.found <- vector(mode = 'logical', length = length(x = symbols))
@@ -878,6 +985,27 @@ GroupCorrelation <- function(
   return(object)
 }
 
+#' Load the Annoy index file
+#'
+#' @param object Neighbor object
+#' @param file Path to file with annoy index
+#'
+#' @return Returns the Neighbor object with the index stored
+#' @export
+#' @concept utilities
+#'
+LoadAnnoyIndex <- function(object, file){
+  metric <- slot(object = object, name = "alg.info")$metric
+  ndim <- slot(object = object, name = "alg.info")$ndim
+  if (is.null(x = metric)) {
+    stop("Provided Neighbor object wasn't generated with annoy")
+  }
+  annoy.idx <- CreateAnn(name = metric, ndim = ndim)
+  annoy.idx$load(path.expand(path = file))
+  Index(object = object) <- annoy.idx
+  return(object)
+}
+
 #' Calculate the variance to mean ratio of logged values
 #'
 #' Calculate the variance to mean ratio (VMR) in non-logspace (return answer in
@@ -976,6 +1104,24 @@ MinMax <- function(data, min, max) {
   return(data2)
 }
 
+#' Calculate the percentage of a vector above some threshold
+#'
+#' @param x Vector of values
+#' @param threshold Threshold to use when calculating percentage
+#'
+#' @return Returns the percentage of \code{x} values above the given threshold
+#'
+#' @export
+#' @concept utilities
+#'
+#' @examples
+#' set.seed(42)
+#' PercentAbove(sample(1:100, 10), 75)
+#'
+PercentAbove <- function(x, threshold) {
+  return(length(x = x[x > threshold]) / length(x = x))
+}
+
 #' Calculate the percentage of all counts that belong to a given set of features
 #'
 #' This function enables you to easily calculate the percentage of all the counts belonging to a
@@ -1193,18 +1339,11 @@ PseudobulkExpression <- function(
     if (slot[1] == 'scale.data') {
       na.matrix <- data.return[[1]]
       na.matrix[1:length(x = na.matrix)] <- NA
-      # TODO: restore once check.matrix is in SeuratObject
-      # toRet <- CreateSeuratObject(
-      #   counts = na.matrix,
-      #   project = if (pb.method == "average") "Average" else "Aggregate",
-      #   assay = names(x = data.return)[1],
-      #   check.matrix = FALSE,
-      #   ...
-      # )
       toRet <- CreateSeuratObject(
         counts = na.matrix,
         project = if (pb.method == "average") "Average" else "Aggregate",
         assay = names(x = data.return)[1],
+        check.matrix = FALSE,
         ...
       )
       toRet <- SetAssayData(
@@ -1226,18 +1365,11 @@ PseudobulkExpression <- function(
         new.data = data.return[[1]]
       )
     } else {
-      # TODO: restore once check.matrix is in SeuratObject
-      # toRet <- CreateSeuratObject(
-      #   counts = data.return[[1]],
-      #   project = if (pb.method == "average") "Average" else "Aggregate",
-      #   assay = names(x = data.return)[1],
-      #   check.matrix = FALSE,
-      #   ...
-      # )
       toRet <- CreateSeuratObject(
         counts = data.return[[1]],
         project = if (pb.method == "average") "Average" else "Aggregate",
         assay = names(x = data.return)[1],
+        check.matrix = FALSE,
         ...
       )
       toRet <- SetAssayData(
@@ -1253,9 +1385,7 @@ PseudobulkExpression <- function(
         if (slot[i] == 'scale.data') {
           na.matrix <- data.return[[i]]
           na.matrix[1:length(x = na.matrix)] <- NA
-          # TODO: restore once check.matrix is in SeuratObject
-          # toRet[[names(x = data.return)[i]]] <- CreateAssayObject(counts = na.matrix, check.matrix = FALSE)
-          toRet[[names(x = data.return)[i]]] <- CreateAssayObject(counts = na.matrix)
+          toRet[[names(x = data.return)[i]]] <- CreateAssayObject(counts = na.matrix, check.matrix = FALSE)
           toRet <- SetAssayData(
             object = toRet,
             assay = names(x = data.return)[i],
@@ -1275,9 +1405,7 @@ PseudobulkExpression <- function(
             new.data = as.matrix(x = data.return[[i]])
           )
         } else {
-          # TODO: restore once check.matrix is in SeuratObject
-          # toRet[[names(x = data.return)[i]]] <- CreateAssayObject(counts = data.return[[i]], check.matrix = FALSE)
-          toRet[[names(x = data.return)[i]]] <- CreateAssayObject(counts = data.return[[i]])
+          toRet[[names(x = data.return)[i]]] <- CreateAssayObject(counts = data.return[[i]], check.matrix = FALSE)
           toRet <- SetAssayData(
             object = toRet,
             assay = names(x = data.return)[i],
@@ -1307,46 +1435,6 @@ PseudobulkExpression <- function(
   }
 }
 
-#' Load the Annoy index file
-#'
-#' @param object Neighbor object
-#' @param file Path to file with annoy index
-#'
-#' @return Returns the Neighbor object with the index stored
-#' @export
-#' @concept utilities
-#'
-LoadAnnoyIndex <- function(object, file){
-  metric <- slot(object = object, name = "alg.info")$metric
-  ndim <- slot(object = object, name = "alg.info")$ndim
-  if (is.null(x = metric)) {
-    stop("Provided Neighbor object wasn't generated with annoy")
-  }
-  annoy.idx <- CreateAnn(name = metric, ndim = ndim)
-  annoy.idx$load(path.expand(path = file))
-  Index(object = object) <- annoy.idx
-  return(object)
-}
-
-#' Save the Annoy index
-#'
-#' @param object A Neighbor object with the annoy index stored
-#' @param file Path to file to write index to
-#'
-#' @export
-#' @concept utilities
-#'
-SaveAnnoyIndex <- function(
-  object,
-  file
-) {
-  index <- Index(object = object)
-  if (is.null(x = index)) {
-    stop("Index for provided Neighbor object is NULL")
-  }
-  index$save(path.expand(path = file))
-}
-
 #' Regroup idents based on meta.data info
 #'
 #' For cells in each ident, set a new identity based on the most common value
@@ -1378,9 +1466,63 @@ RegroupIdents <- function(object, metadata) {
   return(object)
 }
 
+#' Save the Annoy index
+#'
+#' @param object A Neighbor object with the annoy index stored
+#' @param file Path to file to write index to
+#'
+#' @export
+#' @concept utilities
+#'
+SaveAnnoyIndex <- function(
+  object,
+  file
+) {
+  index <- Index(object = object)
+  if (is.null(x = index)) {
+    stop("Index for provided Neighbor object is NULL")
+  }
+  index$save(path.expand(path = file))
+}
+
+#' Find the Quantile of Data
+#'
+#' Converts a quantile in character form to a number regarding some data.
+#' String form for a quantile is represented as a number prefixed with
+#' \dQuote{q}; for example, 10th quantile is \dQuote{q10} while 2nd quantile is
+#' \dQuote{q2}. Will only take a quantile of non-zero data values
+#'
+#' @param cutoff The cutoff to turn into a quantile
+#' @param data The data to turn find the quantile of
+#'
+#' @return The numerical representation of the quantile
+#'
+#' @importFrom stats quantile
+#'
+#' @export
+#' @concept utilities
+#'
+#' @examples
+#' set.seed(42)
+#' SetQuantile('q10', sample(1:100, 10))
+#'
+SetQuantile <- function(cutoff, data) {
+  if (grepl(pattern = '^q[0-9]{1,2}$', x = as.character(x = cutoff), perl = TRUE)) {
+    this.quantile <- as.numeric(x = sub(
+      pattern = 'q',
+      replacement = '',
+      x = as.character(x = cutoff)
+    )) / 100
+    data <- unlist(x = data)
+    data <- data[data > 0]
+    cutoff <- quantile(x = data, probs = this.quantile)
+  }
+  return(as.numeric(x = cutoff))
+}
+
 #' @rdname UpdateSymbolList
 #'
-#' @return For \code{UpdateSymbolList}, \code{symbols} with updated symbols from
+#' @return \code{UpdateSymbolList}: \code{symbols} with updated symbols from
 #' HGNC's gene names database
 #'
 #' @export
@@ -1402,6 +1544,7 @@ UpdateSymbolList <- function(
     symbols = symbols,
     timeout = timeout,
     several.ok = several.ok,
+    search.types = 'prev_symbol',
     verbose = verbose,
     ...
   ))
@@ -1449,7 +1592,7 @@ as.data.frame.Matrix <- function(
   row.names = NULL,
   optional = FALSE,
   ...,
-  stringsAsFactors = default.stringsAsFactors()
+  stringsAsFactors = getOption(x = "stringsAsFactors", default = FALSE)
 ) {
   return(as.data.frame(
     x = as.matrix(x = x),
@@ -1464,40 +1607,86 @@ as.data.frame.Matrix <- function(
 # Internal
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-# Set a default value if an object is null
-#
-# @param lhs An object to set if it's null
-# @param rhs The value to provide if x is null
-#
-# @return rhs if lhs is null, else lhs
-#
-# @author Hadley Wickham
-# @references https://adv-r.hadley.nz/functions.html#missing-arguments
-#
-`%||%` <- function(lhs, rhs) {
-  if (!is.null(x = lhs)) {
-    return(lhs)
-  } else {
-    return(rhs)
-  }
+.AsList <- function(x) {
+  x <- as.list(x = x)
+  return(sapply(
+    X = unique(x = names(x = x)),
+    FUN = function(i) {
+      return(unlist(
+        x = x[which(x = names(x = x) == i)],
+        recursive = FALSE,
+        use.names = FALSE
+      ))
+    },
+    simplify = FALSE,
+    USE.NAMES = TRUE
+  ))
 }
 
-# Set a default value if an object is NOT null
-#
-# @param lhs An object to set if it's NOT null
-# @param rhs The value to provide if x is NOT null
-#
-# @return lhs if lhs is null, else rhs
-#
-# @author Hadley Wickham
-# @references https://adv-r.hadley.nz/functions.html#missing-arguments
-#
-`%iff%` <- function(lhs, rhs) {
-  if (!is.null(x = lhs)) {
-    return(rhs)
-  } else {
-    return(lhs)
+#' @importFrom ggplot2 cut_number
+#'
+.Cut <- function(min, max, n) {
+  breaks <- levels(x = cut_number(x = c(min, max), n = n))
+  breaks <- gsub(pattern = '.*,', replacement = '', x = breaks)
+  breaks <- gsub(pattern = ']$', replacement = '', x = breaks)
+  as.numeric(x = breaks)
+}
+
+.FindE <- function(x) {
+  x <- as.character(x = x)
+  if (grepl(pattern = 'e', x = x)) {
+    return(as.integer(x = gsub(pattern = '.*e', replacement = '', x = x)))
+  } else if (grepl(pattern = '^0\\.', x = x)) {
+    x <- unlist(x = strsplit(
+      x = gsub(pattern = '.*\\.', replacement = '', x = x),
+      split = ''
+    ))
+    idx <- which(x = x != '0')
+    return(-idx)
   }
+  stop("Invalid format")
+}
+
+#' @importFrom SeuratObject Boundaries
+#'
+.BoundariesByImage <- function(object, fov, boundaries) {
+  if (!is.list(x = boundaries)) {
+    if (is.null(x = names(x = boundaries))) {
+      boundaries <- rep_len(x = list(boundaries), length.out = length(x = fov))
+      names(x = boundaries) <- fov
+    } else {
+      boundaries <- .AsList(x = boundaries)
+    }
+  }
+  if (any(!nchar(x = names(x = boundaries)))) {
+    missing <- setdiff(x = fov, y = names(x = boundaries))
+    idx <- which(x = !nchar(x = names(x = boundaries)))
+    boundaries <- c(
+      boundaries[intersect(x = names(x = boundaries), y = fov)],
+      rep_len(x = boundaries[idx], length.out = length(x = missing))
+    )
+    names(x = boundaries)[!nchar(x = names(x = boundaries))] <- missing
+  }
+  if (any(!fov %in% names(x = boundaries))) {
+    for (i in setdiff(x = fov, y = names(x = boundaries))) {
+      boundaries[[i]] <- Boundaries(object = object[[i]])[1L]
+    }
+  }
+  fov <- union(x = fov, y = names(x = boundaries))
+  if (length(x = boundaries) != length(x = fov)) {
+    fov <- intersect(x = fov, y = names(x = boundaries))
+  }
+  boundaries <- boundaries[fov]
+  for (i in fov) {
+    boundaries[[i]] <- Filter(
+      f = function(x) {
+        return(x %in% Boundaries(object = object[[i]]) || is_na(x = x))
+      },
+      x = boundaries[[i]]
+    )
+  }
+  boundaries <- Filter(f = length, x = boundaries)
+  return(boundaries)
 }
 
 # Generate chunk points
@@ -1726,22 +1915,19 @@ CheckDuplicateCellNames <- function(object.list, verbose = TRUE, stop = FALSE) {
 
 
 # Create an empty dummy assay to replace existing assay
-#
+#' @importFrom Matrix sparseMatrix
 CreateDummyAssay <- function(assay) {
-  cm <- as.sparse(x = matrix(
-    data = 0,
-    nrow = nrow(x = assay),
-    ncol = ncol(x = assay)
-  ))
+  cm <- sparseMatrix(
+    i = {},
+    j = {},
+    dims = c(nrow(x = assay), ncol(x = assay))
+  )
+  cm <- as.sparse(x = cm)
   rownames(x = cm) <- rownames(x = assay)
   colnames(x = cm) <- colnames(x = assay)
-  # TODO: restore once check.matrix is in SeuratObject
-  # return(CreateAssayObject(
-  #   counts = cm,
-  #   check.matrix = FALSE
-  # ))
   return(CreateAssayObject(
-    counts = cm
+    counts = cm,
+    check.matrix = FALSE
   ))
 }
 
@@ -1984,14 +2170,14 @@ ModifyParam <- function(param, value) {
   env2[[param]] <- value
 }
 
-# Give hints for old paramters and their newer counterparts
+# Give hints for old parameters and their newer counterparts
 #
 # This is a non-exhaustive list. If your function isn't working properly based
 # on the parameters you give it, please read the documentation for your function
 #
-# @param param A vector of paramters to get hints for
+# @param param A vector of parameters to get hints for
 #
-# @return Parameter hints for the specified paramters
+# @return Parameter hints for the specified parameters
 #
 OldParamHints <- function(param) {
   param.conversion <- c(
@@ -2017,34 +2203,43 @@ OldParamHints <- function(param) {
   return(param.conversion[param])
 }
 
-# Check the existence of a package
+# Check if a web resource is available
 #
-# @param ... Package names
-# @param error If true, throw an error if the package doesn't exist
+# @param url A URL
+# @param strict Perform a strict web availability test
+# @param seconds Timeout in seconds
 #
-# @return Invisibly returns boolean denoting if the package is installed
+# @return \code{TRUE} if \url{is available} otherwise \code{FALSE}
 #
-PackageCheck <- function(..., error = TRUE) {
-  pkgs <- unlist(x = c(...), use.names = FALSE)
-  package.installed <- vapply(
-    X = pkgs,
-    FUN = requireNamespace,
-    FUN.VALUE = logical(length = 1L),
-    quietly = TRUE
-  )
-  if (error && any(!package.installed)) {
-    stop(
-      "Cannot find the following packages: ",
-      paste(pkgs[!package.installed], collapse = ', '),
-      ". Please install"
-    )
+#' @importFrom httr GET status_code timeout
+#
+# @keywords internal
+#
+Online <- function(url, strict = FALSE, seconds = 5L) {
+  if (isTRUE(x = strict)) {
+    code <- 200L
+    comp <- identical
+  } else {
+    code <- 404L
+    comp <- Negate(f = identical)
   }
-  invisible(x = package.installed)
+  request <- tryCatch(
+    expr = GET(url = url, timeout(seconds = seconds)),
+    error = function(err) {
+      code <- if (grepl(pattern = '^Timeout was reached', x = err$message)) {
+        408L
+      } else {
+        404L
+      }
+      return(code)
+    }
+  )
+  return(comp(x = status_code(x = request), y = code))
 }
 
 # Parenting parameters from one environment to the next
 #
-# This function allows one to modifiy a parameter in a parent environement
+# This function allows one to modify a parameter in a parent environment
 # The primary use of this is to ensure logging functions store correct parameters
 # if they've been modified by a child function or method
 #
@@ -2090,17 +2285,6 @@ Parenting <- function(parent.find = 'Seurat', ...) {
   }
 }
 
-# Calculate the percentage of a vector above some threshold
-#
-# @param x Vector of values
-# @param threshold Threshold to use when calculating percentage
-#
-# @return Returns the percentage of `x` values above the given threshold
-#
-PercentAbove <- function(x, threshold) {
-  return(length(x = x[x > threshold]) / length(x = x))
-}
-
 # Generate a random name
 #
 # Make a name from randomly sampled lowercase letters,
@@ -2144,6 +2328,71 @@ RemoveLastField <- function(string, delim = "_") {
   }
 }
 
+# Calculate row mean of a sparse matrix
+# @param mat sparse matrix
+# @return A vector of row mean
+#
+RowMeanSparse <- function(mat) {
+  mat <- RowSparseCheck(mat = mat)
+  output <- row_mean_dgcmatrix(
+    x = slot(object = mat, name = "x"),
+    i = slot(object = mat, name = "i"),
+    rows = nrow(x = mat),
+    cols = ncol(x = mat)
+    )
+  names(x = output) <- rownames(x = mat)
+  return(output)
+}
+
+# Calculate row sum of a sparse matrix
+#
+# @param mat sparse matrix
+# @return A vector of row sum
+#
+RowSumSparse <- function(mat) {
+  mat <- RowSparseCheck(mat = mat)
+  output <- row_sum_dgcmatrix(
+    x = slot(object = mat, name = "x"),
+    i = slot(object = mat, name = "i"),
+    rows = nrow(x = mat),
+    cols = ncol(x = mat)
+  )
+  names(x = output) <- rownames(x = mat)
+  return(output)
+}
+
+# Calculate row variance of a sparse matrix
+#
+# @param mat sparse matrix
+# @return A vector of row variance
+#
+RowVarSparse <- function(mat) {
+  mat <- RowSparseCheck(mat = mat)
+  output <- row_var_dgcmatrix(
+    x = slot(object = mat, name = "x"),
+    i = slot(object = mat, name = "i"),
+    rows = nrow(x = mat),
+    cols = ncol(x = mat)
+  )
+  names(x = output) <- rownames(x = mat)
+  return(output)
+}
+
+# Check if the input matrix is dgCMatrix
+#
+# @param mat sparse matrix
+# @return A dgCMatrix
+#
+RowSparseCheck <- function(mat) {
+  if (!inherits(x = mat, what = "sparseMatrix")) {
+    stop("Input should be sparse matrix")
+  } else if (!is(object = mat, class2 = "dgCMatrix")) {
+    warning("Input matrix is converted to dgCMatrix.")
+    mat <- as.sparse(x = mat)
+  }
+  return(mat)
+}
+
 # Sweep out array summaries
 #
 # Reimplmentation of \code{\link[base]{sweep}} to maintain compatability with
