@@ -2,7 +2,7 @@
 set.seed(42)
 
 pbmc.file <- system.file('extdata', 'pbmc_raw.txt', package = 'Seurat')
-pbmc.test <- as(as.matrix(read.table(pbmc.file, sep = "\t", row.names = 1)), "dgCMatrix")
+pbmc.test <- as.sparse(x = as.matrix(read.table(pbmc.file, sep = "\t", row.names = 1)))
 
 # Tests for object creation (via CreateSeuratObject)
 # --------------------------------------------------------------------------------
@@ -137,7 +137,7 @@ object <- ScaleData(object = object, features = rownames(x = object), verbose =
 
 test_that("split.by option works", {
   expect_equal(GetAssayData(object = object, slot = "scale.data")[, Cells(x = g1)],
-               GetAssayData(object = g1, slot = "scale.data"))  
+               GetAssayData(object = g1, slot = "scale.data"))
   expect_equal(GetAssayData(object = object, slot = "scale.data")[, Cells(x = g2)],
                GetAssayData(object = g2, slot = "scale.data"))
 })
@@ -147,7 +147,7 @@ g2 <- ScaleData(object = g2, features = rownames(x = g2), vars.to.regress = "nCo
 object <- ScaleData(object = object, features = rownames(x = object), verbose = FALSE, split.by = "group", vars.to.regress = "nCount_RNA")
 test_that("split.by option works with regression", {
   expect_equal(GetAssayData(object = object, slot = "scale.data")[, Cells(x = g1)],
-               GetAssayData(object = g1, slot = "scale.data"))  
+               GetAssayData(object = g1, slot = "scale.data"))
   expect_equal(GetAssayData(object = object, slot = "scale.data")[, Cells(x = g2)],
                GetAssayData(object = g2, slot = "scale.data"))
 })
@@ -214,7 +214,7 @@ downsampled.umis <- SampleUMI(
 )
 downsampled.umis.p.cell <- SampleUMI(
   data = GetAssayData(object = object, slot = "counts"),
-  max.umi = seq(50, 840, 10),
+  max.umi = seq(50, 1640, 20),
   verbose = FALSE,
   upsample = TRUE
 )
@@ -238,18 +238,18 @@ object <- FindVariableFeatures(object = object, selection.method = "mean.var.plo
 test_that("mean.var.plot selection option returns expected values", {
   expect_equal(VariableFeatures(object = object)[1:4], c("PTGDR", "SATB1", "ZNF330", "S100B"))
   expect_equal(length(x = VariableFeatures(object = object)), 20)
-  expect_equal(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$mean[1:2], c(8.328927, 8.444462), tolerance = 1e6)
-  expect_equal(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$dispersion[1:2], c(10.552507, 10.088223), tolerance = 1e6)
-  expect_equal(as.numeric(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$dispersion.scaled[1:2]), c(0.1113214, -0.1113214), tolerance = 1e6)
+  expect_equal(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$mean[1:2], c(8.328927, 8.444462), tolerance = 1e-6)
+  expect_equal(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$dispersion[1:2], c(10.552507, 10.088223), tolerance = 1e-6)
+  expect_equal(as.numeric(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$dispersion.scaled[1:2]), c(0.1113214, -0.1332181523), tolerance = 1e-6)
 })
 
 object <- FindVariableFeatures(object, selection.method = "dispersion", verbose = FALSE)
 test_that("dispersion selection option returns expected values", {
   expect_equal(VariableFeatures(object = object)[1:4], c("PCMT1", "PPBP", "LYAR", "VDAC3"))
   expect_equal(length(x = VariableFeatures(object = object)), 230)
-  expect_equal(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$mean[1:2], c(8.328927, 8.444462), tolerance = 1e6)
-  expect_equal(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$dispersion[1:2], c(10.552507, 10.088223), tolerance = 1e6)
-  expect_equal(as.numeric(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$dispersion.scaled[1:2]), c(0.1113214, -0.1113214), tolerance = 1e6)
+  expect_equal(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$mean[1:2], c(8.328927, 8.444462), tolerance = 1e-6)
+  expect_equal(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$dispersion[1:2], c(10.552507, 10.088223), tolerance = 1e-6)
+  expect_equal(as.numeric(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')$dispersion.scaled[1:2]), c(0.1113214, -0.1332181523), tolerance = 1e-6)
   expect_true(!is.unsorted(rev(HVFInfo(object = object[["RNA"]], selection.method = 'mvp')[VariableFeatures(object = object), "dispersion"])))
 })
 
@@ -257,9 +257,9 @@ object <- FindVariableFeatures(object, selection.method = "vst", verbose = FALSE
 test_that("vst selection option returns expected values", {
   expect_equal(VariableFeatures(object = object)[1:4], c("PPBP", "IGLL5", "VDAC3", "CD1C"))
   expect_equal(length(x = VariableFeatures(object = object)), 230)
-  expect_equal(unname(object[["RNA"]][["vst.variance", drop = TRUE]][1:2]), c(1.0251582, 1.2810127), tolerance = 1e6)
-  expect_equal(unname(object[["RNA"]][["vst.variance.expected", drop = TRUE]][1:2]), c(1.1411616, 2.7076228), tolerance = 1e6)
-  expect_equal(unname(object[["RNA"]][["vst.variance.standardized", drop = TRUE]][1:2]), c(0.8983463, 0.4731134), tolerance = 1e6)
+  expect_equal(unname(object[["RNA"]][["vst.variance", drop = TRUE]][1:2]), c(1.0251582, 1.2810127), tolerance = 1e-6)
+  expect_equal(unname(object[["RNA"]][["vst.variance.expected", drop = TRUE]][1:2]), c(1.1411616, 2.7076228), tolerance = 1e-6)
+  expect_equal(unname(object[["RNA"]][["vst.variance.standardized", drop = TRUE]][1:2]), c(0.8983463, 0.4731134), tolerance = 1e-6)
   expect_true(!is.unsorted(rev(object[["RNA"]][["vst.variance.standardized", drop = TRUE]][VariableFeatures(object = object)])))
 })
 
@@ -288,39 +288,43 @@ test_that("CustomNormalize works as expected", {
 
 # Tests for SCTransform
 # ------------------------------------------------------------------------------
+context("SCTransform")
 object <- suppressWarnings(SCTransform(object = object, verbose = FALSE))
 
 test_that("SCTransform wrapper works as expected", {
   expect_true("SCT" %in% names(object))
-  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "scale.data"))[1]), 13.33038640)
+  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "scale.data"))[1]), 11.40288448)
   expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], slot = "scale.data"))[5]), 0)
-  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "data"))[1]), 55.29678, tolerance = 1e6)
-  expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], slot = "data"))[5]), 11.74404, tolerance = 1e6)
-  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "counts"))[1]), 123)
+  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "data"))[1]), 57.7295742, tolerance = 1e-6)
+  expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], slot = "data"))[5]), 11.74403719, tolerance = 1e-6)
+  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "counts"))[1]), 129)
   expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], slot = "counts"))[5]), 28)
   expect_equal(length(VariableFeatures(object[["SCT"]])), 220)
-  expect_equal(object[["SCT"]][[]]["MS4A1", "sct.detection_rate"], 0.15)
-  expect_equal(object[["SCT"]][[]]["MS4A1", "sct.gmean"], 0.2027364, tolerance = 1e6)
-  expect_equal(object[["SCT"]][[]]["MS4A1", "sct.variance"], 1.025158, tolerance = 1e6)
-  expect_equal(object[["SCT"]][[]]["MS4A1", "sct.residual_mean"], 0.2512783, tolerance = 1e6)
-  expect_equal(object[["SCT"]][[]]["MS4A1", "sct.residual_variance"], 3.551259, tolerance = 1e6)
+  fa <- SCTResults(object = object, assay = "SCT", slot = "feature.attributes")
+  expect_equal(fa["MS4A1", "detection_rate"], 0.15)
+  expect_equal(fa["MS4A1", "gmean"], 0.2027364, tolerance = 1e-6)
+  expect_equal(fa["MS4A1", "variance"], 1.025158, tolerance = 1e-6)
+  expect_equal(fa["MS4A1", "residual_mean"], 0.2362887, tolerance = 1e-6)
+  expect_equal(fa["MS4A1", "residual_variance"], 2.875761, tolerance = 1e-6)
 })
 
-object <- suppressWarnings(SCTransform(object = object, ncells = 40, verbose = FALSE))
+suppressWarnings(RNGversion(vstr = "3.5.0"))
+object <- suppressWarnings(SCTransform(object = object, ncells = 40, verbose = FALSE, seed.use =  42))
 test_that("SCTransform ncells param works", {
   expect_true("SCT" %in% names(object))
-  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "scale.data"))[1]), 11.834969847)
+  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "scale.data"))[1]), 12.02126, tolerance = 1e6)
   expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], slot = "scale.data"))[5]), 0)
-  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "data"))[1]), 55.29678, tolerance = 1e6)
-  expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], slot = "data"))[5]), 11.74404, tolerance = 1e6)
-  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "counts"))[1]), 121)
-  expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], slot = "counts"))[5]), 25)
+  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "data"))[1]), 60.65299, tolerance = 1e-6)
+  expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], slot = "data"))[5]), 11.74404, tolerance = 1e-6)
+  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], slot = "counts"))[1]), 136)
+  expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], slot = "counts"))[5]), 28)
   expect_equal(length(VariableFeatures(object[["SCT"]])), 220)
-  expect_equal(object[["SCT"]][[]]["MS4A1", "sct.detection_rate"], 0.15)
-  expect_equal(object[["SCT"]][[]]["MS4A1", "sct.gmean"], 0.2027364, tolerance = 1e6)
-  expect_equal(object[["SCT"]][[]]["MS4A1", "sct.variance"], 1.025158, tolerance = 1e6)
-  expect_equal(object[["SCT"]][[]]["MS4A1", "sct.residual_mean"], 0.2512783, tolerance = 1e6)
-  expect_equal(object[["SCT"]][[]]["MS4A1", "sct.residual_variance"], 3.551259, tolerance = 1e6)
+  fa <- SCTResults(object = object, assay = "SCT", slot = "feature.attributes")
+  expect_equal(fa["MS4A1", "detection_rate"], 0.15)
+  expect_equal(fa["MS4A1", "gmean"], 0.2027364, tolerance = 1e-6)
+  expect_equal(fa["MS4A1", "variance"], 1.025158, tolerance = 1e-6)
+  expect_equal(fa["MS4A1", "residual_mean"], 0.2829672, tolerance = 1e-3)
+  expect_equal(fa["MS4A1", "residual_variance"], 3.674079, tolerance = 1e-3)
 })
 
 suppressWarnings(object[["SCT_SAVE"]] <- object[["SCT"]])
